#ifndef DESCRIPTOR_TABLES_H
#define DESCRIPTOR_TABLES_H

#include "os.h"

void write_tss(int num, USHORT ss0, ULONG esp0);

// asm functions in flush.asm
extern void gdt_flush(ULONG);
extern void tss_flush();
extern void idt_flush(ULONG);

// Initialisation function is publicly accessible.
void init_descriptor_tables();

// Allows the kernel stack in the TSS to be changed.
void set_kernel_stack(ULONG stack);

/* Defines a GDT entry */
struct gdt_entry
{
    unsigned short limit_low;
    unsigned short base_low;
    unsigned char  base_middle;
    unsigned char  access;
    unsigned char  granularity;
    unsigned char  base_high;
}__attribute__((packed));

struct gdt_ptr
{
    USHORT limit;
    ULONG   base;
}__attribute__((packed));

typedef struct gdt_entry gdt_entry_t;
typedef struct gdt_ptr   gdt_ptr_t;

// IDT entry
struct idt_entry
{
    USHORT base_lo;
    USHORT sel;
    UCHAR always0;
    UCHAR flags;
    USHORT base_hi;
}__attribute__((packed)); //prevent compiler optimization

struct idt_ptr
{
    USHORT limit;
    ULONG  base;
}__attribute__((packed)); //prevent compiler optimization

typedef struct idt_entry idt_entry_t;
typedef struct idt_ptr   idt_ptr_t;

// Task State Segment (TSS)
struct tss_entry
{
    USHORT prev_tss;
    USHORT reserved10;

    ULONG  esp0;

    USHORT ss0;
    USHORT reserved09;

    ULONG  esp1;

    USHORT ss1;
    USHORT reserved08;

    ULONG  esp2;

    USHORT ss2;
    USHORT reserved07;

    ULONG  cr3;
    ULONG  eip;
    ULONG  eflags;
    ULONG  eax;
    ULONG  ecx;
    ULONG  edx;
    ULONG  ebx;
    ULONG  esp;
    ULONG  ebp;
    ULONG  esi;
    ULONG  edi;

    USHORT es;
    USHORT reserved06;

    USHORT cs;
    USHORT reserved05;

    USHORT ss;
    USHORT reserved04;

    USHORT ds;
    USHORT reserved03;

    USHORT fs;
    USHORT reserved02;

    USHORT gs;
    USHORT reserved01;

    USHORT ldt;
    USHORT reserved00;

    USHORT trap; //bit0 only! bit1...15 reserved
    USHORT iomap_base;

} __attribute__((packed));

typedef struct tss_entry tss_entry_t;

#endif
